import unittest
from .GeometricSequence import Geometric


class TestGeometricSequence(unittest.TestCase):

    def test_nth_term(self):
        """Тест вычисления n-го члена последовательности"""
        sequence = Geometric(a1=2, q=3)
        self.assertEqual(sequence.nth_term(1), 2)
        self.assertEqual(sequence.nth_term(2), 6)
        self.assertEqual(sequence.nth_term(3), 18)
        self.assertEqual(sequence.nth_term(5), 162)

    def test_sum_of_terms(self):
        """Тест вычисления суммы первых n членов"""
        sequence = Geometric(a1=2, q=3)
        self.assertAlmostEqual(sequence.sum_of_terms(1), 2)
        self.assertAlmostEqual(sequence.sum_of_terms(2), 8)
        self.assertAlmostEqual(sequence.sum_of_terms(3), 26)
        self.assertAlmostEqual(sequence.sum_of_terms(5), 242)

    def test_sum_of_terms_q_equals_1(self):
        """Тест суммы для последовательности, где q = 1"""
        sequence = Geometric(a1=5, q=1)
        self.assertEqual(sequence.sum_of_terms(3), 15)
        self.assertEqual(sequence.sum_of_terms(10), 50)

    def test_is_member(self):
        """Тест проверки принадлежности числа последовательности"""
        sequence = Geometric(a1=2, q=3)
        self.assertTrue(sequence.is_member(2))    # Первый член
        self.assertTrue(sequence.is_member(6))    # Второй член
        self.assertTrue(sequence.is_member(162))  # Пятый член
        self.assertFalse(sequence.is_member(50))  # Не принадлежит

    def test_invalid_n_for_nth_term(self):
        """Тест исключения для n < 1 при вычислении n-го члена"""
        sequence = Geometric(a1=2, q=3)
        with self.assertRaises(ValueError):
            sequence.nth_term(0)
        with self.assertRaises(ValueError):
            sequence.nth_term(-1)

    def test_invalid_n_for_sum_of_terms(self):
        """Тест исключения для n < 1 при вычислении суммы"""
        sequence = Geometric(a1=2, q=3)
        with self.assertRaises(ValueError):
            sequence.sum_of_terms(0)
        with self.assertRaises(ValueError):
            sequence.sum_of_terms(-1)

    def test_zero_denominator(self):
        """Тест исключения при q = 0"""
        with self.assertRaises(ValueError):
            Geometric(a1=2, q=0)

    def test_is_member_invalid_cases(self):
        """Тест проверки на принадлежность для отрицательных и нулевых значений"""
        sequence = Geometric(a1=2, q=3)
        self.assertFalse(sequence.is_member(-6))  # Отрицательное число
        self.assertFalse(sequence.is_member(0))   # Ноль не может быть членом последовательности


if __name__ == "__main__":
    unittest.main()



import math


class Quadratic:
    """
    Класс для решения квадратных уравнений вида ax^2 + bx + c = 0.
    """

    def __init__(self, a, b, c):
        """
        Инициализация коэффициентов уравнения.
        :param a: Коэффициент при x^2
        :param b: Коэффициент при x
        :param c: Свободный член
        """
        self.a = a
        self.b = b
        self.c = c

    def calculate_discriminant(self):
        """
        Вычисляет дискриминант уравнения.
        :return: Значение дискриминанта
        """
        return self.b ** 2 - 4 * self.a * self.c

    def solve(self):
        """
        Решает уравнение на основе коэффициентов.
        :return: Кортеж с корнями уравнения или сообщение о решении
        """
        if self.a == 0:
            if self.b == 0:
                return "Нет решений" if self.c != 0 else "Бесконечно много решений"
            return f"Линейное уравнение, корень: {-self.c / self.b}"

        D = self.calculate_discriminant()

        if D > 0:
            # Два вещественных корня
            x1 = (-self.b + math.sqrt(D)) / (2 * self.a)
            x2 = (-self.b - math.sqrt(D)) / (2 * self.a)
            return x1, x2
        elif D == 0:
            # Один (двукратный) корень
            x = -self.b / (2 * self.a)
            return x,
        else:
            # Комплексные корни
            real_part = -self.b / (2 * self.a)
            imaginary_part = math.sqrt(-D) / (2 * self.a)
            return (real_part + imaginary_part * 1j, real_part - imaginary_part * 1j)


import unittest
from .QuadraticSolver import Quadratic


class TestQuadratic(unittest.TestCase):

    def test_two_real_roots(self):
        """Тест уравнения с двумя вещественными корнями: x^2 - 3x + 2 = 0"""
        solver = Quadratic(1, -3, 2)
        roots = solver.solve()
        self.assertAlmostEqual(roots[0], 2.0)
        self.assertAlmostEqual(roots[1], 1.0)

    def test_one_real_root(self):
        """Тест уравнения с одним вещественным корнем: x^2 + 2x + 1 = 0"""
        solver = Quadratic(1, 2, 1)
        roots = solver.solve()
        self.assertAlmostEqual(roots[0], -1.0)

    def test_complex_roots(self):
        """Тест уравнения с комплексными корнями: x^2 + x + 1 = 0"""
        solver = Quadratic(1, 1, 1)
        roots = solver.solve()
        self.assertAlmostEqual(roots[0].real, -0.5)
        self.assertAlmostEqual(roots[0].imag, 0.8660254037844386)
        self.assertAlmostEqual(roots[1].real, -0.5)
        self.assertAlmostEqual(roots[1].imag, -0.8660254037844386)

    def test_linear_equation(self):
        """Тест линейного уравнения: 2x - 4 = 0"""
        solver = Quadratic(0, 2, -4)
        result = solver.solve()
        self.assertEqual(result, "Линейное уравнение, корень: 2.0")

    def test_no_solution(self):
        """Тест уравнения без решений: 0x + 0 = 5"""
        solver = Quadratic(0, 0, 5)
        result = solver.solve()
        self.assertEqual(result, "Нет решений")

    def test_infinite_solutions(self):
        """Тест уравнения с бесконечным числом решений: 0x + 0 = 0"""
        solver = Quadratic(0, 0, 0)
        result = solver.solve()
        self.assertEqual(result, "Бесконечно много решений")

    def test_discriminant_calculation(self):
        """Тест вычисления дискриминанта"""
        solver = Quadratic(1, -3, 2)
        discriminant = solver.calculate_discriminant()
        self.assertEqual(discriminant, 1)

    def test_negative_discriminant(self):
        """Тест дискриминанта для уравнения с комплексными корнями"""
        solver = Quadratic(1, 1, 1)
        discriminant = solver.calculate_discriminant()
        self.assertEqual(discriminant, -3)


if __name__ == "__main__":
    unittest.main()



import math


class Geometric:
    """
    Класс для работы с геометрической последовательностью.
    Геометрическая последовательность определяется первым членом a1 и знаменателем q.
    """
    def __init__(self, a1, q):
        """
        Инициализация геометрической последовательности.
        :param a1: Первый член последовательности
        :param q: Знаменатель последовательности
        """
        if q == 0:
            raise ValueError("Знаменатель q не может быть равен 0")
        self.a1 = a1
        self.q = q

    def nth_term(self, n):
        """
        Вычисляет n-й член последовательности.
        :param n: Номер члена (n >= 1)
        :return: Значение n-го члена
        """
        if n < 1:
            raise ValueError("Номер члена должен быть >= 1")
        return self.a1 * (self.q ** (n - 1))

    def sum_of_terms(self, n):
        """
        Вычисляет сумму первых n членов последовательности.
        :param n: Количество членов (n >= 1)
        :return: Сумма первых n членов
        """
        if n < 1:
            raise ValueError("Количество членов должно быть >= 1")
        if self.q == 1:
            # Если q = 1, все члены равны a1
            return self.a1 * n
        return self.a1 * (1 - self.q ** n) / (1 - self.q)

    def is_member(self, x):
        """
        Проверяет, принадлежит ли число последовательности.
        :param x: Число, которое нужно проверить
        :return: True, если принадлежит, иначе False
        """
        if x == 0:
            return self.a1 == 0  # Если первый член 0, тогда 0 принадлежит последовательности
        # Проверка на принадлежность через логарифмы: x = a1 * q^k
        ratio = x / self.a1
        if ratio <= 0:
            return False
        k = math.log(ratio, self.q)
        return k.is_integer()


from setuptools import setup

setup(name='geo_qua',
      version='0.0',
      description='Geometric sequence and quadratic equation',
      packages=['geo_qua'],
      author_email='olegisupov542@gmail.com',
      zip_safe=False)
